﻿using UnityEngine;
using System.Collections;


public class PlayerController : MonoBehaviour
{
	public float moveDuration = 0.1f;
	public float afterDamagePause = 0.5f;
	public float afterDeathPause = 10.0f;
	public Material healthPickupMaterial;

	public AudioClip[] hurtSounds;

	public UnityEngine.UI.Text healthText;
	public UnityEngine.UI.Text scoreText;
	public UnityEngine.UI.Text deadText;

	private float m_Health = 100.0f;
	private int m_Score = 0;
	private float m_AfterDamageLeft;
	private float m_AfterDeadLeft;
	private int m_OldHighScore;
	private bool m_Cheater;


	private MonoBehaviour _glitchFx;
	private CharacterMove _move;
	private static PlayerController s_Instance;

	public static PlayerController instance { get { return s_Instance; } }

	public void Awake()
	{
		if (s_Instance != null)
			Debug.LogError("Player instance already exists!");
		s_Instance = this;
	}

	void Start ()
	{
		m_OldHighScore = PlayerPrefs.GetInt ("HighScore");
		_move = GetComponent<CharacterMove>();
		_glitchFx = Camera.main.GetComponent<GlitchEffect>();
		if (_glitchFx)
			_glitchFx.enabled = false;
		UpdateHealthUI();
		UpdateScoreUI();
	}
	
	void Update ()
	{
		KeyboardControl ();

		if (m_AfterDeadLeft > 0)
		{
			m_AfterDeadLeft -= Time.deltaTime;
			if (m_AfterDeadLeft < 0)
			{
				// restart game
				Application.LoadLevel (Application.loadedLevel);
			}
		}
		else if (m_AfterDamageLeft > 0)
		{
			m_AfterDamageLeft -= Time.deltaTime;
			if (m_AfterDamageLeft < 0)
			{
				if (_glitchFx)
					_glitchFx.enabled = false;
				m_AfterDamageLeft = 0;
			}
		}
	}

	public bool AddHealth(float amount)
	{
		if (m_AfterDeadLeft > 0)
			return false;
		if (m_Health >= 100.0f)
			return false;
		m_Health += amount;
		m_Health = Mathf.Min (m_Health, 100.0f);
		UpdateHealthUI();
		return true;
	}

	public void AddScore(int score)
	{
		m_Score += score;
		UpdateScoreUI();
	}

	void UpdateScoreUI()
	{
		var str = "Score: " + m_Score;
		if (m_Cheater)
		{
			str = "Score: cheater!";
		}
		else
		{
			if (m_Score > m_OldHighScore)
			{
				PlayerPrefs.SetInt ("HighScore", m_Score);
				PlayerPrefs.Save();
			}
		}
		if (m_OldHighScore > 0)
		{
			str += " (high: " + m_OldHighScore + ")";
		}
		scoreText.text = str;
	}

	void UpdateHealthUI()
	{
		if (healthText)
		{
			if (m_Health <= 0.0f)
				healthText.text = "Health: nope";
			else
				healthText.text = "Health: " + (int)m_Health;
		}
		if (healthPickupMaterial)
		{
			if (m_Health >= 100.0f)
			{
				healthPickupMaterial.SetFloat ("_Desaturate", 0.7f);
				healthPickupMaterial.color = new Color(1,1,1,0.5f);
			}
			else
			{
				healthPickupMaterial.SetFloat ("_Desaturate", 0.0f);
				healthPickupMaterial.color = new Color(1,1,1,1);
			}
		}
	}

	public void DoDamage(float amount)
	{
		if (m_AfterDamageLeft > 0 || m_AfterDeadLeft > 0)
			return;

		m_Health -= amount;
		m_AfterDamageLeft = afterDamagePause;

		AudioSource.PlayClipAtPoint(hurtSounds[Random.Range (0,hurtSounds.Length)], transform.position);

		if (_glitchFx)
			_glitchFx.enabled = true;

		if (m_Health <= 0.0f)
		{
			m_Health = 0.0f;
			GameOver();
		}
		UpdateHealthUI();
	}

	public void GameOver()
	{
		m_AfterDeadLeft = afterDeathPause;
		if (deadText)
		{
			deadText.enabled = true;
		}
	}

	void KeyboardControl()
	{
		if (m_AfterDeadLeft > 0.0f || m_AfterDamageLeft > 0.0f)
			return;
		if (!_move.IsMoving())
		{
			float inputHor = Input.GetAxis ("Horizontal");
			float inputVer = Input.GetAxis ("Vertical");
			if (Mathf.Abs (inputHor) > Mathf.Abs (inputVer))
			{
				// horizontal move
				if (Mathf.Abs (inputHor) > 0.2f)
				{
					_move.Move (Mathf.Sign (inputHor), 0, moveDuration);
				}
			}
			else
			{
				// vertical move
				if (Mathf.Abs (inputVer) > 0.2f)
				{
					_move.Move (0, Mathf.Sign (inputVer), moveDuration);
				}
			}
		}
		if (Input.GetButtonDown("Jump"))
			WeaponSystem.instance.FireActiveWeapon(transform.position, Vector3.one); //@TODO: direction
		if (Input.GetKeyDown (KeyCode.O))
		{
			WeaponSystem.instance.PickupWeapon(WeaponKind.Bass, 10);
			m_Score = 0;
			m_Cheater = true;
			UpdateScoreUI();
		}
	}
}
